/*

	Umberto Rustichelli aka Ubi, 2019, 2020 and the bright future

	esempio di codice per il client del servizio Lambda

	il server e' l'appliance lambda-PeS / alpha-PeS / Timbro Digitale
*/

import net.rustichelli.security.*;

public class LambdaExample
{
	public static void main(String[] args)
	{

		// conterranno, rispettivamente, il contenuto PDF originale
		// e il medesimo contenuto codificato in Base64
		byte [] content;
		String b64input;


		try
		{
			// acquisisci il contenuto del file PDF e codificalo in Base64
			content = b64enc.readFL("testData/input.pdf");
			b64input = b64enc.uc_b64encode(content);

			System.out.println("INFO: content has been Base64-encoded");


			// crea l'oggetto che si occupa della comunicazione
			HttpsGetPage retriever = new HttpsGetPage();
			retriever.HttpsGetPage(); // e' necessario invocare il costruttore

			System.out.println("INFO: base class created");



			/* comunicazione con mutua autenticazione HTTPS

				il server, come avviene nella maggior parte dei casi, presenta un certificato al client.
				Nella mutua autenticazione, anche il client possiede un certificato (la sua credenziale)
					e deve inviarlo al server (ovvero all'appliance lambda-PeS)
			*/

			// indica quale certificato aspettarsi dal server, cioe' dall'appliance.
			// Impostare anche se (v. oltre) si utilizza retriever.DoTrustAll = true.
			// Parametri: il percorso del file contenente il certificato (del server), la password
			// (i file JKS sono cifrati) e l'alias, cioe' l'identificativo del certificato all'interno del JKS:
			retriever.setAuthServer
			(
					"auth-server/pes-app.02010a-nodomain.jks",
					"pass1234",
					"pes-app"
			);

			// impostare (a true) se non si vuole verificare il certificato del server.
			// Puo' essere utile in fase di sviluppo, specialmente con una JVM "complicata"
			retriever.DoTrustAll = true;

			// indica quali sono le credenziali del client. Il file JKS deve contenere il certificato
			// (Che viene inviato al server) e la chiave privata (che non viene mai inviata).
			// Parametri: percorso del file JKS, password di cifratura, formato del JKS.
			// Quest'ultimo puo' essere "JKS" o "P12", come in genere si evince dall'estensione del file
			retriever.setAuthClient
			(
				"auth-client/democli-201912.30000A.jks",
				"pass1234",
				"JKS"
			);

			System.out.println("INFO: authentication parameters have been set");



			/*
				imposta i parametri applicativi Lambda

				i parametri applicativi sono
					indirizzo del server (dell'appliance)
					porta (quasi sempre 443, default per HTTPS),
					nome della configurazione da utilizzare (e' il nome della configurazione registrata sull'appliance9
					contenuto PDF codificato in Base64
					nome del file PDF, come verra' registrato nel Lambda Store. E' opportuno che tale nome
					  sia univoco o quantomeno parlante, es. <data>-<identificativo univoco transazione>.pdf
			*/
			retriever.setLambdaParameters("pes-cloudasb.gt50.org", 443, "lambda", b64input, "test1.pdf");

			// imposta il percorso del file in cui scrivere la risposta del server.
			// Se le operazioni hanno successo, il file conterra' il PDF firmato con il sigillo Lambda, altrimenti
			//   conterra' un breve testo con un codice d'errore
			retriever.setOutputFileName("testData/lambda-output.pdf");

			System.out.println("INFO: proceeding now!");


			// chiama il servizio, scrivi il file con la risposta
			String rets = retriever.retrieveURL();


			/*
				la stringa restituita in rets indica successo se la comunicazione e' avvenuta,
			      non si entra nel merito del contenuto restituito dal server.
			      Il codice dovrebbe controllare se si tratta effettivamente di un PDF,
			      o almeno se il file e' grande abbastanza. Se e' piccolo, contiene sicuramente
			      un codice d'errore
			*/
			System.out.println("INFO: return string is : " + rets);
		}
		catch (Exception exc)
		{
			System.out.println("ERROR: exception is " + exc.toString());
		}
	}
}

